@@ -325,8 +325,8 @@ tds_char2hex(TDS_CHAR *dest, size_t destlen, const TDS_CHAR * src, size_t srclen
 			if ('A' <= hex1 && hex1 <= 'F') {
 				hex1 -= ('A' - 10);
 			} else {
-				tdsdump_log(TDS_DBG_INFO1,
-					    "error_handler:  attempt to convert data stopped by syntax error in source field \n");
+				tdsdump_log(TDS_DBG_ERROR,
+					    "error_handler: attempt to convert data stopped by syntax error in source field\n");
 				return TDS_CONVERT_SYNTAX;
 			}
 		}
@@ -643,6 +643,88 @@ tds_convert_int8_numeric(unsigned char scale,
 	return tds_numeric_change_prec_scale(&(cr->n), orig_prec, orig_scale);
 }
 
+static const char digits2[200] =
+	"0001020304050607080910111213141516171819202122232425262728293031323334353637383940414243444546474849"
+	"5051525354555657585960616263646566676869707172737475767778798081828384858687888990919293949596979899";
+
+static inline void
+tds02dfast(char out[2], int v)
+{
+	out[0] = digits2[v * 2];
+	out[1] = digits2[v * 2 + 1];
+}
+
+void
+tds_02d_fast(char out[2], int v)
+{
+	tds02dfast(out, v);
+}
+
+/**
+ * Format an unsigned int32 right-justified on a char[10] buffer.
+ * Returns the amount of leading padding.
+ */
+size_t
+tds_u32toa_fast_right(char out[10], uint32_t v)
+{
+	char *p = out + 10;
+
+	/* Fill buffer from the end so it comes out in the right order */
+	while (v >= 100u) {
+		uint32_t q = v / 100u;
+		uint32_t r = v % 100u;
+
+		p -= 2;
+		tds02dfast(p, r);
+		v = q;
+	}
+
+	if (v >= 10u) {
+		p -= 2;
+		tds02dfast(p, v);
+	} else if (v > 0u)
+		*--p = (char) ('0' + v);
+
+	return p - out;
+}
+
+/* Returns number of digits written */
+size_t
+tds_u32toa_fast(char out[10], uint32_t v)
+{
+	char buf[10];
+	size_t pad;
+
+	/* Small case optimizations */
+	if (v < 10u) {
+		*out = (char) ('0' + v);
+		return 1;
+	}
+	if (v < 100u) {
+		tds02dfast(out, v);
+		return 2;
+	}
+
+	pad = tds_u32toa_fast_right(buf, v);
+
+	memcpy(out, buf + pad, 10 - pad);
+	return 10 - pad;
+}
+
+size_t
+tds_i32toa_fast(char out[11], int32_t v)
+{
+	size_t sign_length = 0;
+
+	if (v < 0) {
+		*out++ = '-';
+		sign_length = 1;
+		if (TDS_LIKELY(v != (-2147483647 - 1)))
+			v = -v;
+	}
+	return tds_u32toa_fast(out, v) + sign_length;
+}
+
 static TDS_INT
 tds_convert_int(TDS_INT num, int desttype, CONV_RESULT * cr)
 {
@@ -651,7 +733,7 @@ tds_convert_int(TDS_INT num, int desttype, CONV_RESULT * cr)
 	switch (desttype) {
 	case TDS_CONVERT_CHAR:
 	case CASE_ALL_CHAR:
-		sprintf(tmp_str, "%d", num);
+		tmp_str[tds_i32toa_fast(tmp_str, num)] = 0;
 		return string_to_result(desttype, tmp_str, cr);
 		break;
 	case SYBSINT1:
@@ -898,9 +980,15 @@ tds_convert_uint8(const TDS_UINT8 *src, int desttype, CONV_RESULT * cr)
 }
 
 static TDS_INT
-tds_convert_numeric(const TDS_NUMERIC * src, int desttype, CONV_RESULT * cr)
+tds_convert_numeric(const TDS_NUMERIC *src, int desttype, CONV_RESULT *cr)
 {
-	char tmpstr[MAXPRECISION];
+	union
+	{
+		/* if the number has precision == scale == MAXPRECISION and it's negative a "-0." is prefixed
+		 * to the digits. Also account for terminator and possible invalid out of range number. */
+		char tmpstr[MAXPRECISION + 5];
+		TDS_NUMERIC num;
+	} u;
 	TDS_INT i, ret;
 	TDS_UINT ui;
 	TDS_INT8 bi;
@@ -908,112 +996,112 @@ tds_convert_numeric(const TDS_NUMERIC * src, int desttype, CONV_RESULT * cr)
 	switch (desttype) {
 	case TDS_CONVERT_CHAR:
 	case CASE_ALL_CHAR:
-		if (tds_numeric_to_string(src, tmpstr) < 0)
+		if (tds_numeric_to_string(src, u.tmpstr) < 0)
 			return TDS_CONVERT_FAIL;
-		return string_to_result(desttype, tmpstr, cr);
+		return string_to_result(desttype, u.tmpstr, cr);
 		break;
 	case SYBSINT1:
-		cr->n = *src;
-		ret = tds_numeric_change_prec_scale(&(cr->n), 3, 0);
+		u.num = *src;
+		ret = tds_numeric_change_prec_scale(&u.num, 3, 0);
 		if (ret < 0)
 			return ret;
-		if (cr->n.array[1])
+		if (u.num.array[1])
 			return TDS_CONVERT_OVERFLOW;
-		i = cr->n.array[2];
-		if (cr->n.array[0])
+		i = u.num.array[2];
+		if (u.num.array[0])
 			i = -i;
-		if (i != 0 && ((i >> 7) ^ cr->n.array[0]) & 1)
+		if (i != 0 && ((i >> 7) ^ u.num.array[0]) & 1)
 			return TDS_CONVERT_OVERFLOW;
 		cr->ti = (TDS_TINYINT) i;
 		return sizeof(TDS_TINYINT);
 		break;
 	case SYBINT1:
 	case SYBUINT1:
-		cr->n = *src;
-		ret = tds_numeric_change_prec_scale(&(cr->n), 3, 0);
+		u.num = *src;
+		ret = tds_numeric_change_prec_scale(&(u.num), 3, 0);
 		if (ret < 0)
 			return ret;
-		if (cr->n.array[1] || (cr->n.array[0] && cr->n.array[2]))
+		if (u.num.array[1] || (u.num.array[0] && u.num.array[2]))
 			return TDS_CONVERT_OVERFLOW;
-		cr->ti = cr->n.array[2];
+		cr->ti = u.num.array[2];
 		return sizeof(TDS_TINYINT);
 		break;
 	case SYBINT2:
-		cr->n = *src;
-		ret = tds_numeric_change_prec_scale(&(cr->n), 5, 0);
+		u.num = *src;
+		ret = tds_numeric_change_prec_scale(&(u.num), 5, 0);
 		if (ret < 0)
 			return ret;
-		if (cr->n.array[1])
+		if (u.num.array[1])
 			return TDS_CONVERT_OVERFLOW;
-		i = TDS_GET_UA2BE(&(cr->n.array[2]));
-		if (cr->n.array[0])
+		i = TDS_GET_UA2BE(&(u.num.array[2]));
+		if (u.num.array[0])
 			i = -i;
-		if (i != 0 && ((i >> 15) ^ cr->n.array[0]) & 1)
+		if (i != 0 && ((i >> 15) ^ u.num.array[0]) & 1)
 			return TDS_CONVERT_OVERFLOW;
 		cr->si = (TDS_SMALLINT) i;
 		return sizeof(TDS_SMALLINT);
 		break;
 	case SYBUINT2:
-		cr->n = *src;
-		ret = tds_numeric_change_prec_scale(&(cr->n), 5, 0);
+		u.num = *src;
+		ret = tds_numeric_change_prec_scale(&(u.num), 5, 0);
 		if (ret < 0)
 			return ret;
-		i = TDS_GET_UA2BE(&(cr->n.array[2]));
-		if ((i != 0 && cr->n.array[0]) || cr->n.array[1])
+		i = TDS_GET_UA2BE(&(u.num.array[2]));
+		if ((i != 0 && u.num.array[0]) || u.num.array[1])
 			return TDS_CONVERT_OVERFLOW;
 		cr->usi = (TDS_USMALLINT) i;
 		return sizeof(TDS_USMALLINT);
 		break;
 	case SYBINT4:
-		cr->n = *src;
-		ret = tds_numeric_change_prec_scale(&(cr->n), 10, 0);
+		u.num = *src;
+		ret = tds_numeric_change_prec_scale(&(u.num), 10, 0);
 		if (ret < 0)
 			return ret;
-		if (cr->n.array[1])
+		if (u.num.array[1])
 			return TDS_CONVERT_OVERFLOW;
-		i = (TDS_INT) TDS_GET_UA4BE(&(cr->n.array[2]));
-		if (cr->n.array[0])
+		i = (TDS_INT) TDS_GET_UA4BE(&(u.num.array[2]));
+		if (u.num.array[0])
 			i = -i;
-		if (i != 0 && ((i >> 31) ^ cr->n.array[0]) & 1)
+		if (i != 0 && ((i >> 31) ^ u.num.array[0]) & 1)
 			return TDS_CONVERT_OVERFLOW;
 		cr->i = i;
 		return sizeof(TDS_INT);
 		break;
 	case SYBUINT4:
-		cr->n = *src;
-		ret = tds_numeric_change_prec_scale(&(cr->n), 10, 0);
+		u.num = *src;
+		ret = tds_numeric_change_prec_scale(&(u.num), 10, 0);
 		if (ret < 0)
 			return ret;
-		ui = TDS_GET_UA4BE(&(cr->n.array[2]));
-		if ((ui != 0 && cr->n.array[0]) || cr->n.array[1])
+		ui = TDS_GET_UA4BE(&(u.num.array[2]));
+		if ((ui != 0 && u.num.array[0]) || u.num.array[1])
 			return TDS_CONVERT_OVERFLOW;
 		cr->ui = ui;
 		return sizeof(TDS_UINT);
 		break;
 	case SYBINT8:
-		cr->n = *src;
-		ret = tds_numeric_change_prec_scale(&(cr->n), 20, 0);
+		u.num = *src;
+		ret = tds_numeric_change_prec_scale(&(u.num), 20, 0);
 		if (ret < 0)
 			return ret;
-		if (cr->n.array[1])
+		if (u.num.array[1])
 			return TDS_CONVERT_OVERFLOW;
-		bi = TDS_GET_UA4BE(&(cr->n.array[2]));
-		bi = (bi << 32) + TDS_GET_UA4BE(&(cr->n.array[6]));
-		if (cr->n.array[0])
+		bi = TDS_GET_UA4BE(&(u.num.array[2]));
+		bi = (bi << 32) + TDS_GET_UA4BE(&(u.num.array[6]));
+		if (u.num.array[0])
 			bi = -bi;
-		if (bi != 0 && ((bi >> 63) ^ cr->n.array[0]) & 1)
+		if (bi != 0 && ((bi >> 63) ^ u.num.array[0]) & 1)
 			return TDS_CONVERT_OVERFLOW;
 		cr->bi = bi;
 		return sizeof(TDS_INT8);
 		break;
 	case SYBUINT8:
-		cr->n = *src;
-		ret = tds_numeric_change_prec_scale(&(cr->n), 20, 0);
+		u.num = *src;
+		ret = tds_numeric_change_prec_scale(&(u.num), 20, 0);
 		if (ret < 0)
 			return ret;
-		bi = TDS_GET_UA4BE(&(cr->n.array[2]));
-		bi = (bi << 32) + TDS_GET_UA4BE(&(cr->n.array[6]));
-		if ((bi != 0 && cr->n.array[0]) || cr->n.array[1])
+		bi = TDS_GET_UA4BE(&(u.num.array[2]));
+		bi = (bi << 32) + TDS_GET_UA4BE(&(u.num.array[6]));
+		if ((bi != 0 && u.num.array[0]) || u.num.array[1])
 			return TDS_CONVERT_OVERFLOW;
 		cr->ubi = bi;
 		return sizeof(TDS_UINT8);
@@ -1029,32 +1117,32 @@ tds_convert_numeric(const TDS_NUMERIC * src, int desttype, CONV_RESULT * cr)
 		return sizeof(TDS_TINYINT);
 		break;
 	case SYBMONEY4:
-		cr->n = *src;
-		ret = tds_numeric_change_prec_scale(&(cr->n), 10, 4);
+		u.num = *src;
+		ret = tds_numeric_change_prec_scale(&(u.num), 10, 4);
 		if (ret < 0)
 			return ret;
-		if (cr->n.array[1])
+		if (u.num.array[1])
 			return TDS_CONVERT_OVERFLOW;
-		i = (TDS_INT) TDS_GET_UA4BE(&(cr->n.array[2]));
-		if (cr->n.array[0])
+		i = (TDS_INT) TDS_GET_UA4BE(&(u.num.array[2]));
+		if (u.num.array[0])
 			i = -i;
-		if (i != 0 && ((i >> 31) ^ cr->n.array[0]) & 1)
+		if (i != 0 && ((i >> 31) ^ u.num.array[0]) & 1)
 			return TDS_CONVERT_OVERFLOW;
 		cr->m4.mny4 = i;
 		return sizeof(TDS_MONEY4);
 		break;
 	case SYBMONEY:
-		cr->n = *src;
-		ret = tds_numeric_change_prec_scale(&(cr->n), 20, 4);
+		u.num = *src;
+		ret = tds_numeric_change_prec_scale(&(u.num), 20, 4);
 		if (ret < 0)
 			return ret;
-		if (cr->n.array[1])
+		if (u.num.array[1])
 			return TDS_CONVERT_OVERFLOW;
-		bi = TDS_GET_UA4BE(&(cr->n.array[2]));
-		bi = (bi << 32) + TDS_GET_UA4BE(&(cr->n.array[6]));
-		if (cr->n.array[0])
+		bi = TDS_GET_UA4BE(&(u.num.array[2]));
+		bi = (bi << 32) + TDS_GET_UA4BE(&(u.num.array[6]));
+		if (u.num.array[0])
 			bi = -bi;
-		if (bi != 0 && ((bi >> 63) ^ cr->n.array[0]) & 1)
+		if (bi != 0 && ((bi >> 63) ^ u.num.array[0]) & 1)
 			return TDS_CONVERT_OVERFLOW;
 		cr->m.mny = bi;
 		return sizeof(TDS_MONEY);
@@ -1063,20 +1151,21 @@ tds_convert_numeric(const TDS_NUMERIC * src, int desttype, CONV_RESULT * cr)
 	case SYBDECIMAL:
 		{
 			unsigned char prec = cr->n.precision, scale = cr->n.scale;
+
 			cr->n = *src;
 			return tds_numeric_change_prec_scale(&(cr->n), prec, scale);
 		}
 		break;
 	case SYBFLT8:
-		if (tds_numeric_to_string(src, tmpstr) < 0)
+		if (tds_numeric_to_string(src, u.tmpstr) < 0)
 			return TDS_CONVERT_FAIL;
-		cr->f = atof(tmpstr);
+		cr->f = atof(u.tmpstr);
 		return 8;
 		break;
 	case SYBREAL:
-		if (tds_numeric_to_string(src, tmpstr) < 0)
+		if (tds_numeric_to_string(src, u.tmpstr) < 0)
 			return TDS_CONVERT_FAIL;
-		cr->r = (TDS_REAL) atof(tmpstr);
+		cr->r = (TDS_REAL) atof(u.tmpstr);
 		return 4;
 		break;
 		/* conversions not allowed */
@@ -1916,6 +2005,11 @@ tds_convert_to_binary(int srctype, const TDS_CHAR * src, TDS_UINT srclen, int de
 		len = sizeof(TDS_UNIQUE);
 		break;
 	case CASE_ALL_CHAR:
+		/* skip leading spaces and tabs */
+		while (srclen > 0 && (src[0] == ' ' || src[0] == '\t')) {
+			src++;
+			srclen--;
+		}
 
 		/* skip leading "0x" or "0X" */
 		if (srclen >= 2 && src[0] == '0' && (src[1] == 'x' || src[1] == 'X')) {
@@ -1953,20 +2047,27 @@ tds_convert_to_binary(int srctype, const TDS_CHAR * src, TDS_UINT srclen, int de
 
 /**
  * tds_convert
- * convert a type to another.
- * If you convert to SYBDECIMAL/SYBNUMERIC you MUST initialize precision 
- * and scale of cr.
+ * converts a type to another.
+ * @p srctype and @p desttype should be SYB* TDS types, or, in case of @p desttype you can use
+ * TDS_CONVERT_CHAR/TDS_CONVERT_BINARY. Nullable types are not supported, use tds_get_conversion_type
+ * to get a not-nullable type.
+ *
+ * If you convert to SYBDECIMAL/SYBNUMERIC you MUST initialize precision and scale of cr->n.
+ *
+ * In case of fixed type you can pass directly the pointer to the value.
+ *
  * Do not expect strings to be zero terminated. Databases support zero inside
  * string. Using strlen may result on data loss or even a segmentation fault.
  * Instead, use memcpy to copy destination using length returned.
- * This function does not handle NULL, srclen should be >0.  Client libraries handle NULLs each in their own way. 
+ *
+ * This function does not handle NULL, @p srclen should be >0.  Client libraries handle NULLs each in their own way.
  * @param tds_ctx  context (used in conversion to data and to return messages)
  * @param srctype  type of source
  * @param src      pointer to source data to convert
  * @param srclen   length in bytes of source (not counting terminator or strings)
  * @param desttype type of destination
  * @param cr       structure to hold result
- * @return length of result or TDS_CONVERT_* failure code on failure. All TDS_CONVERT_* constants are <0.
+ * @return length of result or TDS_CONVERT_* failure code on failure. All TDS_CONVERT_* error constants are <0.
  */
 TDS_INT
 tds_convert(const TDSCONTEXT *tds_ctx, int srctype, const void *src, TDS_UINT srclen, int desttype, CONV_RESULT *cr)
@@ -2070,7 +2171,7 @@ tds_convert(const TDSCONTEXT *tds_ctx, int srctype, const void *src, TDS_UINT sr
 	case SYBNTEXT:
 	case SYBMSTABLE:
 	default:
-		return TDS_CONVERT_NOAVAIL;
+		length = TDS_CONVERT_NOAVAIL;
 		break;
 	}
 
@@ -3063,8 +3164,9 @@ two_digit(char *out, int num)
 		num = 1;
 	if (num > 31)
 		num = 31;
-	out[0] = num < 10 ? ' ' : num/10 + '0';
-	out[1] = num%10 + '0';
+	tds02dfast(out, num);
+	if (out[0] == '0')
+		out[0] = ' ';
 }
 
 /**
@@ -3109,7 +3211,7 @@ tds_strftime(char *buf, size_t maxsize, const char *format, const TDSDATEREC * d
 #endif
 
 	/* more characters are required because we replace %z with up to 7 digits */
-	our_format = tds_new(char, strlen(format) + 1 + 5 + 1);
+	our_format = tds_new(char, strlen(format) + (1 + 5 + 1));
 	if (!our_format)
 		return 0;
 
@@ -3164,9 +3266,15 @@ tds_strftime(char *buf, size_t maxsize, const char *format, const TDSDATEREC * d
 
 			--pz;
 			if (prec || pz <= our_format || pz[-1] != '.') {
-				char buf[12];
-
-				sprintf(buf, "%010d", dr->decimicrosecond & 0x7fffffff);
+				char buf[10];
+				/* decimicrosecond should be a number from 0 to 9,999,999
+				 * (regardless of prec, which is just the requested
+				 * display precision). In case we are erroneously passed a
+				 * sign, discard the sign. If erroneously given a number
+				 * too big, discard leading digits.
+				 */
+				memset(buf, '0', sizeof(buf));
+				tds_u32toa_fast_right(buf, dr->decimicrosecond & 0x7fffffff);
 				memcpy(pz, buf + 3, prec);
 				strcpy(pz + prec, format + (pz - our_format) + z_opt_len);
 				pz += prec;
